@@ -7,7 +7,7 @@ use strict;
 
 package XML::Compile::SOAP;  #!!!
 use vars '$VERSION';
-$VERSION = '0.66';
+$VERSION = '0.67';
 
 use Log::Report 'xml-compile-soap', syntax => 'SHORT';
 use List::Util qw/min first/;
@@ -210,7 +210,9 @@ sub array($$$@)
     my $type   = $self->prefixed($itemtype)."$nested\[$size]";
 
     $el->setAttribute(id => $opts{id}) if defined $opts{id};
-    $el->setAttribute($self->prefixed($encns, 'arrayType'), $type);
+    my $at     = $opts{array_type} ? $opts{arrayType} 
+               : $self->prefixed($encns, 'arrayType');
+    $el->setAttribute($at, $type) if defined $at;
 
     if($sparse)
     {   my $placeition = $self->prefixed($encns, 'position');
@@ -376,6 +378,21 @@ sub dec(@)
 
 sub _dec_reader($@)
 {   my ($self, $type) = @_;
+    return $self->{dec}{$type} if $self->{dec}{$type};
+
+    my ($typens, $typelocal) = unpack_type $type;
+    my $schemans  = $self->schemaNS;
+
+    if(   $typens ne $schemans
+       && !$self->schemas->namespaces->find(element => $type))
+    {   # work-around missing element
+        $self->schemas->importDefinitions(<<__FAKE_SCHEMA);
+<schema xmlns="$schemans" targetNamespace="$typens" xmlns:d="$typens">
+<element name="$typelocal" type="d:$typelocal" />
+</schema>
+__FAKE_SCHEMA
+    }
+
     $self->{dec}{$type} ||= $self->schemas->compile
       (READER => $type, @{$self->{dec}{reader_opts}}, @_);
 }
@@ -478,10 +495,8 @@ sub _dec_other($$)
              $data  = { $local => $dec_childs } if $dec_childs;
          }
          else
-         {   $data =
-               { $local => $node->textContent
-               , _TYPE  => $type
-               };
+         {   $data->{$local} = $node->textContent;
+             $data->{_TYPE}  = $basetype if $basetype;
          }
     }
     else
